precision mediump float;
uniform vec2 iResolution;
uniform float iTime;           // shader playback time (in seconds)
uniform float iRandom;
uniform float iSoundData;
uniform float iBass;
uniform float iScale;

#define WAVES 8.0
#define PIE 3.14159
#define TWOPIE 6.28318

vec3 nrand3( vec2 co ){
	vec3 a = fract( cos( co.x*8.3e-3 + co.y )*vec3(1.3e5, 4.7e5, 2.9e5) );
	vec3 b = fract( sin( co.x*0.3e-3 + co.y )*vec3(8.1e5, 1.0e5, 0.1e5) );
	vec3 c = mix(a, b, 0.5);
	return c;
}

void main() {

	vec2 uv = -1.0 + 2.0 * gl_FragCoord.xy / iResolution;

	float time = iTime * TWOPIE;

	vec3 color = vec3(0.0);

	for (float i=0.0; i<WAVES + 1.0; i++) {
        float freq = (i * 2.0 + 10.0) * 0.1 + (iRandom * 0.02 * (WAVES + 1.0 - i)) + (iBass * 0.05);
        //+ iSoundData * 0.003
        vec2 p = vec2(uv);
		freq *= iScale;

		p.x += i * 0.04 + freq * 0.03;
		p.y += sin((p.x * 10.0 * (i * 0.1 ) + time) ) * cos(p.x * 2.0) * freq * 0.2 * ((i + 1.0) / WAVES);
		//float intensity = abs(0.01 / p.y) * clamp(freq, 0.35, 0.5 + 2.0 * iRandom + abs(iSoundData) * 0.0001);
		float intensity = abs(0.01 / p.y) * clamp(freq, 0.35, 0.5 + 2.0 * abs(iSoundData) * 0.05 + iRandom * 0.05);
		color += vec3(1.0 * intensity * (i / 4.0), 1.3 * intensity, 1.2 * intensity) * (3.0 / WAVES);
	}

    const float nLine = 9.0;
    float colorLine = 0.0;

	// Grid
	for(float i=0.0 ; i<nLine; i++){
		float lx = (i/nLine)*(iResolution.x+10.0);
		float ly = (i/nLine)*(iResolution.y+10.0);
		colorLine += 0.1/abs(gl_FragCoord.x-lx);
		colorLine += 0.1/abs(gl_FragCoord.y-ly);
	}

    colorLine += (iRandom * 0.05);
    colorLine = clamp(0.0, 0.1, colorLine);
    colorLine += (iRandom * 0.05);
    color += colorLine;

    float lxl = (5.0/nLine)*(iResolution.x+10.0);
    float lyl = (5.0/nLine)*(iResolution.y+10.0);
    colorLine += 0.1/abs(gl_FragCoord.x-lxl);
    colorLine += 0.1/abs(gl_FragCoord.y-lyl);

    color += colorLine;

	gl_FragColor = vec4(color, 1.0);
}