package se.hellsoft.syncloud;

import android.app.Activity;
import android.content.Context;
import android.media.MediaPlayer;
import android.media.audiofx.Visualizer;
import android.net.Uri;
import android.opengl.GLSurfaceView;
import android.os.AsyncTask;
import android.os.Bundle;
import android.util.Log;
import android.view.MotionEvent;
import android.view.Window;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;

/**
 * User: Erik Westenius
 * Date: 01/11/13
 * Time: 19:06
 */
public class Hacktivity extends Activity {

    private static final String TAG = "Hacktivity";
    private SlaveSurfaceView mGLView;
    private MediaPlayer mMediaPlayer;
    private Visualizer mVisualizer;
    private AudioCapture mAudioCapture;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        mGLView = new SlaveSurfaceView(this);
        requestWindowFeature(Window.FEATURE_NO_TITLE);
        setContentView(mGLView);
    }

    @Override
    protected void onResume() {
        super.onResume();
        mGLView.onResume();
        new SoundCloudPlayer().doInBackground();
    }

    @Override
    protected void onPause() {
        super.onPause();
        mGLView.onPause();
        if (mMediaPlayer != null)
            mMediaPlayer.release();
        if(mAudioCapture != null){
            mAudioCapture.stop();
            mAudioCapture.release();
        }
    }

    public static String readShader(Context context, final int resourceId) {
        InputStream inputStream = context.getResources().openRawResource(resourceId);
        InputStreamReader inputStreamReader = new InputStreamReader(inputStream);
        BufferedReader bufferedReader = new BufferedReader(inputStreamReader);

        String nextLine;
        final StringBuilder shader = new StringBuilder();

        try {
            while ((nextLine = bufferedReader.readLine()) != null) {
                shader.append(nextLine);
                shader.append('\n');
            }
        } catch (IOException e) {
            e.printStackTrace();
            return null;
        }

        return shader.toString();
    }

    private class SoundCloudPlayer extends AsyncTask<String, Void, Void>{

        public static final String TRACK_URL = "http://api.soundcloud.com/tracks/118153066/stream?client_id=010335534542aa219428a79ad33b2081";

        @Override
        protected Void doInBackground(String... params) {
            mMediaPlayer = new MediaPlayer();

            Uri track = Uri.parse(TRACK_URL);
            try {
                mMediaPlayer.setDataSource(Hacktivity.this, track);
                mMediaPlayer.prepare();
                mMediaPlayer.start();

                mVisualizer = new Visualizer(mMediaPlayer.getAudioSessionId());
                mVisualizer.setCaptureSize(Visualizer.getCaptureSizeRange()[1]);

                mVisualizer.setDataCaptureListener(new Visualizer.OnDataCaptureListener() {

                    @Override
                    public void onWaveFormDataCapture(Visualizer visualizer, byte[] bytes,
                                                      int samplingRate) {
                        // TODO Auto-generated method stub
                        int tmp = ((int)bytes[0] & 0xFF) - 128;
//                        Log.d(TAG, "Visualizer: " + tmp);
                        mGLView.setSoundData(tmp);
                    }

                    @Override
                    public void onFftDataCapture(Visualizer visualizer, byte[] fft,
                                                 int samplingRate) {
                        // TODO Auto-generated method stub
//                        Log.d(TAG, "FFT: " + fft[0]);
                        mGLView.setFFTData(fft);
                    }
                }, Visualizer.getMaxCaptureRate(), true, true);
                mVisualizer.setEnabled(true);
            } catch (IOException e) {
                e.printStackTrace();
            }
            return null;
        }
    }

    private class SlaveSurfaceView extends  GLSurfaceView{

        private SlaveRenderer mRenderer;

        public SlaveSurfaceView(Context context) {
            super(context);
            setEGLContextClientVersion(2);
            mRenderer = new SlaveRenderer(readShader(context, R.raw.slave_shader));
            setRenderer(mRenderer);
            setRenderMode(GLSurfaceView.RENDERMODE_CONTINUOUSLY);
        }

        public void setSoundData(int i){
            if(mRenderer != null)
                mRenderer.setSoundData(i);
        }

        private final float TOUCH_SCALE_FACTOR = 180.0f / 320;
        private float mPreviousX;
        private float mPreviousY;

        @Override
        public boolean onTouchEvent(MotionEvent e) {
            // MotionEvent reports input details from the touch screen
            // and other input controls. In this case, you are only
            // interested in events where the touch position changed.

            float x = e.getX();
            float y = e.getY();

            switch (e.getAction()) {
                case MotionEvent.ACTION_MOVE:

                    float dx = x - mPreviousX;
                    float dy = y - mPreviousY;

                    // reverse direction of rotation above the mid-line
                    if (y > getHeight() / 2) {
                        dx = dx * -1 ;
                    }

                    // reverse direction of rotation to left of the mid-line
                    if (x < getWidth() / 2) {
                        dy = dy * -1 ;
                    }

                    mRenderer.mAngle += (dx + dy) * TOUCH_SCALE_FACTOR;  // = 180.0f / 320
                    requestRender();
            }

            mPreviousX = x;
            mPreviousY = y;
            return true;
        }

        public void setFFTData(byte[] fft) {
            if(mRenderer != null)
                mRenderer.setFFTData(fft);
        }
    }
}
