package se.hellsoft.syncloud;

import android.app.Fragment;
import android.content.Context;
import android.media.MediaPlayer;
import android.media.audiofx.Visualizer;
import android.net.Uri;
import android.opengl.GLSurfaceView;
import android.os.AsyncTask;
import android.os.Bundle;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;

/**
* @author Erik Hellman
*/
public class SlaveFragment extends Fragment {

    private static final String TAG = "SlaveFragment";
    private SlaveSurfaceView mGLView;
    private Visualizer mVisualizer;
    private MediaPlayer mMediaPlayer;
    private SlaveRenderer mRenderer;

    public SlaveFragment() {
    }


    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        mGLView = new SlaveSurfaceView(getActivity());
        return mGLView;
    }

    @Override
    public void onResume() {
        super.onResume();
        mGLView.onResume();
    }

    @Override
    public void onPause() {
        super.onPause();
        mGLView.onPause();
    }

    @Override
    public void onDestroyView() {
        super.onDestroyView();
        if(mVisualizer != null) {
            mVisualizer.release();
        }
        mVisualizer = null;
    }

    public static String readShader(Context context, final int resourceId) {
        InputStream inputStream = context.getResources().openRawResource(resourceId);
        InputStreamReader inputStreamReader = new InputStreamReader(inputStream);
        BufferedReader bufferedReader = new BufferedReader(inputStreamReader);

        String nextLine;
        final StringBuilder shader = new StringBuilder();

        try {
            while ((nextLine = bufferedReader.readLine()) != null) {
                shader.append(nextLine);
                shader.append('\n');
            }
        } catch (IOException e) {
            e.printStackTrace();
            return null;
        }

        return shader.toString();
    }

    public void setMediaPlayer(MediaPlayer mediaPlayer, boolean playing) {
        if (mediaPlayer != null) {
            mMediaPlayer = mediaPlayer;
            if(playing) {
                mRenderer.setScale(3.5f);
            }

            mVisualizer = new Visualizer(mMediaPlayer.getAudioSessionId());
            mVisualizer.setCaptureSize(Visualizer.getCaptureSizeRange()[1]);

            mVisualizer.setDataCaptureListener(new Visualizer.OnDataCaptureListener() {

                @Override
                public void onWaveFormDataCapture(Visualizer visualizer, byte[] bytes,
                                                  int samplingRate) {
                    // TODO Auto-generated method stub
                    int tmp = ((int)bytes[0] & 0xFF) - 128;
                    mGLView.setSoundData(tmp);
                }

                @Override
                public void onFftDataCapture(Visualizer visualizer, byte[] fft,
                                             int samplingRate) {
                    // TODO Auto-generated method stub
    //                        Log.d(TAG, "FFT: " + fft[0]);

                }
            }, Visualizer.getMaxCaptureRate(), true, false);
            mVisualizer.setEnabled(true);
        } else {
            if(!playing) {
                mRenderer.setScale(0.1f);
            }
        }

    }

    private class SlaveSurfaceView extends GLSurfaceView {


        public SlaveSurfaceView(Context context) {
            super(context);
            setEGLContextClientVersion(2);
            mRenderer = new SlaveRenderer(readShader(context, R.raw.slave_shader));
            setRenderer(mRenderer);
            setRenderMode(GLSurfaceView.RENDERMODE_CONTINUOUSLY);
        }

        public void setSoundData(int i){
            if(mRenderer != null)
                mRenderer.setSoundData(i);
        }

        private final float TOUCH_SCALE_FACTOR = 180.0f / 320;
        private float mPreviousX;
        private float mPreviousY;

        @Override
        public boolean onTouchEvent(MotionEvent e) {
            // MotionEvent reports input details from the touch screen
            // and other input controls. In this case, you are only
            // interested in events where the touch position changed.

            float x = e.getX();
            float y = e.getY();

            switch (e.getAction()) {
                case MotionEvent.ACTION_MOVE:

                    float dx = x - mPreviousX;
                    float dy = y - mPreviousY;

                    // reverse direction of rotation above the mid-line
                    if (y > getHeight() / 2) {
                        dx = dx * -1 ;
                    }

                    // reverse direction of rotation to left of the mid-line
                    if (x < getWidth() / 2) {
                        dy = dy * -1 ;
                    }

                    mRenderer.mAngle += (dx + dy) * TOUCH_SCALE_FACTOR;  // = 180.0f / 320
                    requestRender();
            }

            mPreviousX = x;
            mPreviousY = y;
            return true;
        }
    }
}
