package se.hellsoft.syncloud.soundcloud;

import android.content.Context;
import android.graphics.Bitmap;
import android.os.Environment;
import android.os.SystemClock;
import android.util.Log;
import com.android.volley.Cache;
import com.android.volley.RequestQueue;
import com.android.volley.Response;
import com.android.volley.VolleyError;
import com.android.volley.toolbox.*;
import org.json.JSONArray;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

/**
 * @author Erik Hellman
 */
public class SoundCloudAPI implements Response.ErrorListener {
    public static final String TAG = "SoundCloudAPI";
    public static final String SOUNDCLOUD_CLIENT_ID = "010335534542aa219428a79ad33b2081";
    public static final String TRACKS_BASE_URI = "http://api.soundcloud.com/tracks.json";
    private RequestQueue mRequestQueue;
    private ImageLoader.ImageCache mImageCache;

    public SoundCloudAPI() {
    }

    public void init(Context context) {
        mRequestQueue = Volley.newRequestQueue(context);
        mImageCache = new MyImageCache();
        mRequestQueue.start();
    }

    public void destroy() {
        mRequestQueue.stop();
    }

    public void searchTracks(String query, Response.Listener<JSONArray> listener) {
        try {
            String url = TRACKS_BASE_URI + '?';
            url += "q=" + URLEncoder.encode(query, "UTF-8");
            url += "&filter=streamable&client_id=" + SOUNDCLOUD_CLIENT_ID;
            JsonArrayRequest jsonArrayRequest = new JsonArrayRequest(url, listener, this);
            mRequestQueue.add(jsonArrayRequest);
        } catch (UnsupportedEncodingException e) {
            Log.e(TAG, "Error encoding URL.", e);
        }
    }

    public NetworkImageView getNetworkImageView(String url, Context context) {
        NetworkImageView networkImageView = new NetworkImageView(context);
        networkImageView.setImageUrl(url, new ImageLoader(mRequestQueue, mImageCache));
        return networkImageView;
    }

    @Override
    public void onErrorResponse(VolleyError error) {
        // Ignore for now...
    }

    class MyImageCache implements ImageLoader.ImageCache {
        private Map<String, CacheBitmap> mCache;

        MyImageCache() {
            mCache = Collections.synchronizedMap(new HashMap<String, CacheBitmap>());
        }

        @Override
        public Bitmap getBitmap(String url) {
            return mCache.get(url).bitmap;
        }

        @Override
        public void putBitmap(String url, Bitmap bitmap) {
            if (mCache.size() > 20) {
                Map.Entry<String, CacheBitmap> oldest = null;
                for (Map.Entry<String, CacheBitmap> cacheBitmapEntry : mCache.entrySet()) {
                    if(oldest == null) {
                        oldest = cacheBitmapEntry;
                    } else if (oldest.getValue().timestamp > cacheBitmapEntry.getValue().timestamp) {
                        oldest = cacheBitmapEntry;
                    }
                }
                if (oldest != null) {
                    mCache.remove(oldest.getKey());
                }
            }

            mCache.put(url, new CacheBitmap(bitmap, SystemClock.elapsedRealtime()));
        }

        class CacheBitmap {
            public Bitmap bitmap;
            public long timestamp;

            CacheBitmap(Bitmap bitmap, long timestamp) {
                this.bitmap = bitmap;
                this.timestamp = timestamp;
            }
        }
    }
}
